==> dasu_2.02.jar: edu/northwestern/dasu/measurement/DasuHttpModule.java <==
/**
 * Dasu Project
 *
 * File:         DasuHttpModule.java
 * RCS:          $Id: DasuHttpModule.java,v 1.4 2010/06/02 02:14:04 mas939 Exp $
 * Description:  DasuHttpModule class (see below)
 * Author:       Mario Sanchez
 *               Northwestern Systems Research Group
 *               Department of Computer Science
 *               Northwestern University
 * Created:      Apr 15, 2010 at 07:38:30 PM
 * Language:     Java
 * Package:      edu.northwestern.ono.experiment
 * Status:       Experimental (Do Not Distribute)
 *
 * (C) Copyright 2010, Northwestern University, all rights reserved.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 */
package edu.northwestern.dasu.measurement;

import java.io.IOException;
import java.io.InputStream;
import java.net.URL;
import java.net.URLConnection;
import java.util.ArrayList;
import java.util.Hashtable;
import java.util.List;

import edu.northwestern.dasu.MainGeneric;
import edu.northwestern.dasu.drools.FactHttpProbeResult;
import edu.northwestern.dasu.stats.Statistics;
import edu.northwestern.dasu.util.Pair;



/**
 * @author Mario Sanchez &lt;msanchez@u.northwestern.edu&gt;
 * 
 *         The HamecaPingModuleV2 class manages the set of 
 *         Pings performed by the peer.
 */

public class DasuHttpModule extends DasuProbeModule {	
		
	private class TaskEntry {
		
		public Integer taskId;
		public String url;
		
		public TaskEntry(String url, Integer taskId) {
			this.taskId = taskId;
			this.url = url;
		}
	}
	
	public Hashtable<Long, FactHttpProbeResult> httpResultTable;
	private ArrayList<TaskEntry> toDoUrlList = new ArrayList<TaskEntry>();
	private static DasuHttpModule self;
	private final int chunkSize=1024;

	public DasuHttpModule() {
		super("DasuHttpModule");
		this.pi = MainGeneric.getPluginInterface();
		httpResultTable = new Hashtable<Long, FactHttpProbeResult>();
		
		active = true;
	}
	
	public synchronized static DasuHttpModule getInstance() {
		if (self == null) {
			self = new DasuHttpModule();
			self.setDaemon(true);
			return self;
		} else {
			return self;
		}
	}

	@Override
	public void run() {
		
	  while (!MainGeneric.isShuttingDown() && isActive()) {
		 
			TaskEntry taskEntry = null;
			synchronized (toDoUrlList) {
				while ((toDoUrlList.size()==0) && !MainGeneric.isShuttingDown()) {
					try {
						toDoUrlList.wait(sleepInterval*1000);
					} catch (InterruptedException e) {
						self = null;
						return;
					}
				}
				
				if (toDoUrlList.size() > 0)
					taskEntry = toDoUrlList.remove(0);
				
				if (DEBUG)
					System.out.println("DasuHttpModule: Http server " + taskEntry.url );
			}
			
			if (taskEntry.url.equals("HALT")) 
				return;
			
			FactHttpProbeResult result = (FactHttpProbeResult) probeIp(taskEntry.url, taskEntry.taskId);
			updateHttpResultQueue(result);


	  } // end while active
		
	  self = null;
	  
	}
		
	@Override
	public void probeAll(List<String> list, Integer taskId) {

		synchronized (toDoUrlList) {
			for (String ip : list)
				toDoUrlList.add(new TaskEntry(ip, taskId));
			toDoUrlList.notifyAll();
		}
	}
	
	@Override
	public void probeWithPriority(String url, Integer taskId) {
		
		synchronized (toDoUrlList) {
			toDoUrlList.add(new TaskEntry(url, taskId));
			toDoUrlList.notifyAll();
		}		
	}

	@Override
	public void probe(String url, Integer taskId) {

		synchronized (toDoUrlList) {
			toDoUrlList.add(new TaskEntry(url, taskId));
			toDoUrlList.notifyAll();
		}
	}
	
	
	public FactHttpProbeResult fileUrl(String fAddress, String localFileName, Integer taskId) {

		FactHttpProbeResult ret = new FactHttpProbeResult(fAddress);
		URLConnection  uCon = null;
		Long startTime = 0L;
		Long endTime = 0L;
		
		//String destinationDir =
		//	Main.getPluginInterface().getUtilities().getAzureusUserDir();
		//OutputStream outStream = null;


		InputStream is = null;
		try {
			URL Url;
			byte[] buf;
			int ByteRead,ByteWritten=0;
			Url= new URL(fAddress);
			//outStream = new BufferedOutputStream(new
			//FileOutputStream(destinationDir+"/"+localFileName));

			uCon = Url.openConnection();
			
			startTime = System.currentTimeMillis();
			is = uCon.getInputStream();
			buf = new byte[chunkSize];
			while ((ByteRead = is.read(buf)) != -1) {
				//outStream.write(buf, 0, ByteRead);
				ByteWritten += ByteRead;
			}
			endTime = System.currentTimeMillis();

			float fileSize = ((float) ByteWritten)/1024F/1024F; /*MB*/
			float dldTime = (endTime - startTime)/1000F;
			float dldRate = (((float) ByteWritten)*8)/dldTime/1024;
			
			ret = new FactHttpProbeResult(fAddress, dldTime, dldRate, ByteWritten, taskId);
			Statistics.getInstance().addHttpResultDasu(ret);

			if (DEBUG) {
				System.out.println("Downloaded Successfully.");
				System.out.println("File name:\""+localFileName+ "\"\nNo ofbytes: " + ByteWritten);
				System.out.println("File Size: " + ((float) ByteWritten)/1024F/1024F + " MB");
				System.out.println("Dld Time: " + (endTime - startTime)/1000F + " seconds");
				Float dr = (((float) ByteWritten)*8)/((endTime - startTime)/1000);
				System.out.println("Dld Speed: " +  dr/1024 + " kbps");
			}
			
		} catch (Exception e) {
			System.out.println("HttpModule: Http Timeout: " + fAddress);
			return ret;
			
		} finally {
			try {
				
				if (is != null)
					is.close();
				//outStream.close();
				
			} catch (IOException e) {
				System.out.println("HttpModule: File not found!: " + localFileName);
				return ret;
			}
		}
		return ret;

	}
	
	@Override
	protected Object probeIp(String fAddress, Integer taskId) {
	
		FactHttpProbeResult ret = new FactHttpProbeResult(fAddress);
		int slashIndex =fAddress.lastIndexOf('/');
		int periodIndex =fAddress.lastIndexOf('.');
	
		String fileName=fAddress.substring(slashIndex + 1);
	
		if (periodIndex >=1 &&  slashIndex >= 0 && slashIndex < fAddress.length()-1) {
			ret = fileUrl(fAddress, fileName, taskId);
		}
		else {
			System.err.println("HamecaHttpModule: ERROR: path or file name");
		}

		return ret;
	}
	
	private void updateHttpResultQueue(FactHttpProbeResult result) {
		
		Long id = System.nanoTime();
		httpResultTable.put(id, result);
		
		synchronized(AMCoordinatorV2.events) {
			AMCoordinatorV2.events.add(
				new Pair<Long, EventType>(id, EventType.HHM_RESULT));
			AMCoordinatorV2.events.notifyAll();
		}
	}
	
	@Override
	public void stopModule() {
		
		setActive(false);
		synchronized(toDoUrlList) {
			toDoUrlList.add(new TaskEntry("HALT", -1));
			toDoUrlList.notify();
		}
	
		self = null;
	}
	
}

==> dasu_3.02.jar: edu/northwestern/dasu/measurement/DasuHttpModule.java <==
/**
 * Dasu Project
 *
 * File:         DasuHttpModule.java
 * RCS:          $Id$
 * Description:  DasuHttpModule class (see below)
 * Author:       Mario Sanchez
 *               Northwestern Systems Research Group
 *               Department of Computer Science
 *               Northwestern University
 * Created:      Apr 15, 2010 at 07:38:30 PM
 * Language:     Java
 * Package:      edu.northwestern.ono.experiment
 * Status:       Experimental (Do Not Distribute)
 *
 * (C) Copyright 2010, Northwestern University, all rights reserved.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 */
package edu.northwestern.dasu.measurement;

import java.io.IOException;
import java.io.InputStream;
import java.net.URL;
import java.net.URLConnection;
import java.util.ArrayList;
import java.util.Hashtable;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;
import java.util.logging.Logger;

import edu.northwestern.dasu.Main;
import edu.northwestern.dasu.drools.FactHttpProbeResult;
import edu.northwestern.dasu.stats.Statistics;
import edu.northwestern.dasu.util.Pair;



/**
 * @author Mario Sanchez &lt;msanchez@u.northwestern.edu&gt;
 * 
 *         The HamecaPingModuleV2 class manages the set of 
 *         Pings performed by the peer.
 */

public class DasuHttpModule extends DasuProbeModule {	
		
	private class TaskEntry {
		
		public Integer taskId;
		public String url;
		public Map<String, String> headers;
		public String target;
		
		public TaskEntry(String url, Integer taskId, String target) {
			this.taskId = taskId;
			this.url = url;
			this.target = target;
		}
		
		public TaskEntry(String url, Integer taskId, Map params, String target) {
			this.taskId = taskId;
			this.url = url;
			this.target = target;
						
			if (params != null) {
				
				if (params.containsKey("headers"))
					this.headers = (Map<String, String>) params.get("headers");
				
			}
			
		}
	}
	
	private final static Logger LOGGER = Logger.getLogger(DasuHttpModule.class.getName());
	
	public Hashtable<Long, FactHttpProbeResult> httpResultTable;
	private ArrayList<TaskEntry> toDoUrlList = new ArrayList<TaskEntry>();
	private static DasuHttpModule self;
	private final int chunkSize=1024;

	public DasuHttpModule() {
		super("DasuHttpModule");
		this.pi = Main.getPluginInterface();
		httpResultTable = new Hashtable<Long, FactHttpProbeResult>();
		
		active = true;
	}
	
	public synchronized static DasuHttpModule getInstance() {
		if (self == null) {
			self = new DasuHttpModule();
			self.setDaemon(true);
			return self;
		} else {
			return self;
		}
	}

	@Override
	public void run() {
		
	  while (!Main.isShuttingDown() && isActive()) {
		 
			TaskEntry taskEntry = null;
			synchronized (toDoUrlList) {
				while ((toDoUrlList.size()==0) && !Main.isShuttingDown()) {
					try {
						toDoUrlList.wait(sleepInterval*1000);
					} catch (InterruptedException e) {
						self = null;
						return;
					}
				}
				
				if (toDoUrlList.size() > 0)
					taskEntry = toDoUrlList.remove(0);
				
				LOGGER.info("Http server " + taskEntry.url);
			}
			
			if (taskEntry.url.equals("HALT")) 
				return;
			
			FactHttpProbeResult result = (FactHttpProbeResult) probeIp(taskEntry.url, taskEntry.headers, taskEntry.taskId, taskEntry.target);
			updateHttpResultQueue(result);


	  } // end while active
		
	  self = null;
	  
	}
		
	@Override
	public void probeAll(List<String> list, Integer taskId, String target) {

		synchronized (toDoUrlList) {
			for (String ip : list)
				toDoUrlList.add(new TaskEntry(ip, taskId, target));
			toDoUrlList.notifyAll();
		}
	}
	
	@Override
	public void probeWithPriority(String url, Integer taskId, String target) {
		
		synchronized (toDoUrlList) {
			toDoUrlList.add(new TaskEntry(url, taskId, target));
			toDoUrlList.notifyAll();
		}		
	}

	public void probe(String url, Integer taskId, Map params, String target) {
		
		LOGGER.fine("probe() " + url + " " + taskId + " " + params + " " + target);
		
		synchronized (toDoUrlList) {
			toDoUrlList.add(new TaskEntry(url, taskId, params, target));
			toDoUrlList.notifyAll();
		}

	}
	
	@Override
	public void probe(String url, Integer taskId, String target) {

		synchronized (toDoUrlList) {
			toDoUrlList.add(new TaskEntry(url, taskId, target));
			toDoUrlList.notifyAll();
		}
	}
	
	
	public FactHttpProbeResult fileUrl(String fAddress, String localFileName, Integer taskId, String target, Map<String, String> headers) {

		FactHttpProbeResult ret = new FactHttpProbeResult(fAddress);
		ret.setTarget(target);
		URLConnection  uCon = null;
		Long startTime = 0L;
		Long endTime = 0L;
		
		//String destinationDir =
		//	Main.getPluginInterface().getUtilities().getAzureusUserDir();
		//OutputStream outStream = null;


		InputStream is = null;
		try {
			URL Url;
			byte[] buf;
			int ByteRead,ByteWritten=0;
			Url= new URL(fAddress);
			//outStream = new BufferedOutputStream(new
			//FileOutputStream(destinationDir+"/"+localFileName));

			uCon = Url.openConnection();
			
			if (headers != null) {
				for ( Entry<String, String> e : headers.entrySet() ) {
					uCon.setRequestProperty(e.getKey(), e.getValue());
				}
			}
			
			startTime = System.currentTimeMillis();
			is = uCon.getInputStream();
			buf = new byte[chunkSize];
			while ((ByteRead = is.read(buf)) != -1) {
				//outStream.write(buf, 0, ByteRead);
				ByteWritten += ByteRead;
			}
			endTime = System.currentTimeMillis();

			float fileSize = ((float) ByteWritten)/1024F/1024F; /*MB*/
			float dldTime = (endTime - startTime)/1000F;
			float dldRate = (((float) ByteWritten)*8)/dldTime/1024;
			
			ret = new FactHttpProbeResult(fAddress, dldTime, dldRate, ByteWritten, taskId);
			ret.setTarget(target);
			Statistics.getInstance().addHttpResultDasu(ret);

			LOGGER.fine("Downloaded Successfully.");
			LOGGER.fine("File name:\""+localFileName+ "\"\nNo ofbytes: " + ByteWritten);
			LOGGER.fine("File Size: " + ((float) ByteWritten)/1024F/1024F + " MB");
			LOGGER.fine("Dld Time: " + (endTime - startTime)/1000F + " seconds");
			Float dr = (((float) ByteWritten)*8)/((endTime - startTime)/1000);
			LOGGER.fine("Dld Speed: " +  dr/1024 + " kbps");
			
		} catch (Exception e) {
			LOGGER.info("Http Timeout: " + fAddress);
			return ret;
			
		} finally {
			try {
				
				if (is != null)
					is.close();
				//outStream.close();
				
			} catch (IOException e) {
				LOGGER.info("File not found!: " + localFileName);
				return ret;
			}
		}
		return ret;

	}

	/*default probe uses default headers server*/
	
	@Override
	protected Object probeIp(String fAddress, Integer taskId, String target) {
		return probeIp(fAddress, null, taskId, target);
	}

	
	protected Object probeIp(String fAddress, Map<String, String> headers, Integer taskId, String target) {
	
		FactHttpProbeResult ret = new FactHttpProbeResult(fAddress);
		ret.setTarget(target);
		int slashIndex =fAddress.lastIndexOf('/');
		int periodIndex =fAddress.lastIndexOf('.');
	
		String fileName=fAddress.substring(slashIndex + 1);
	
		if (periodIndex >=1 &&  slashIndex >= 0 && slashIndex < fAddress.length()-1) {
			ret = fileUrl(fAddress, fileName, taskId, target, headers);
		}
		else {
			LOGGER.warning("ERROR: path or file name");
		}

		return ret;
	}
	
	private void updateHttpResultQueue(FactHttpProbeResult result) {
		
		Long id = System.nanoTime();
		httpResultTable.put(id, result);
		
		synchronized(AMCoordinatorV2.events) {
			AMCoordinatorV2.events.add(
				new Pair<Long, EventType>(id, EventType.HHM_RESULT));
			AMCoordinatorV2.events.notifyAll();
		}
	}
	
	@Override
	public void stopModule() {
		
		setActive(false);
		synchronized(toDoUrlList) {
			toDoUrlList.add(new TaskEntry("HALT", -1, null));
			toDoUrlList.notify();
		}
	
		self = null;
	}
	
}
